package net.knarcraft.knargui;

import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.ClickType;
import org.bukkit.event.inventory.InventoryType;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * An abstract GUI used for creating specific GUIs
 */
@SuppressWarnings("unused")
public abstract class AbstractGUI {

    private UUID uuid;
    private Inventory inventory;
    private Map<Integer, Map<ClickType, GUIAction>> actions;

    /**
     * Instantiates a new abstract GUI
     *
     * @param inventorySize <p>The size of the inventory (ignored if inventory type is set)</p>
     * @param inventoryName <p>The name of the inventory</p>
     * @param inventoryType <p>The type of inventory to use for the GUI, or null for a variable chest GUI</p>
     */
    public AbstractGUI(int inventorySize, String inventoryName, InventoryType inventoryType) {
        Inventory inventory;
        if (inventoryType == null) {
            inventory = Bukkit.createInventory(null, inventorySize, inventoryName);
        } else {
            inventory = Bukkit.createInventory(null, inventoryType, inventoryName);
        }
        instantiate(inventory);
    }

    /**
     * Instantiates a new abstract GUI
     *
     * @param inventory <p>The inventory to use for the GUI</p>
     */
    public AbstractGUI(Inventory inventory) {
        instantiate(inventory);
    }

    private void instantiate(Inventory inventory) {
        this.uuid = UUID.randomUUID();
        this.inventory = inventory;
        this.actions = new HashMap<>();
        GUIRegistry.registerGUI(this);
    }

    /**
     * Gets this inventory's unique id
     *
     * @return <p>This inventory's unique id</p>
     */
    public UUID getUuid() {
        return this.uuid;
    }

    /**
     * Gets the inventory used for this GUI
     *
     * @return <p>The inventory used for this GUI</p>
     */
    public Inventory getInventory() {
        return this.inventory;
    }

    /**
     * Sets the item in the given inventory slot
     *
     * @param inventorySlot <p>The inventory slot to set the item for</p>
     * @param itemStack     <p>The item to display in the slot</p>
     * @return <p>This GUI. Used for chaining commands</p>
     */
    public AbstractGUI setItem(int inventorySlot, ItemStack itemStack) {
        this.inventory.setItem(inventorySlot, itemStack);
        return this;
    }

    /**
     * Sets the action for clicking the given item slot
     *
     * @param inventorySlot <p>The inventory slot to set the action for</p>
     * @param clickType     <p>The type of click to set an action for</p>
     * @param action        <p>The action triggered when a click occurs</p>
     * @return <p>This GUI. Used for chaining commands</p>
     */
    public AbstractGUI setClickAction(int inventorySlot, ClickType clickType, GUIAction action) {
        if (!this.actions.containsKey(inventorySlot)) {
            this.actions.put(inventorySlot, new HashMap<>());
        }
        this.actions.get(inventorySlot).put(clickType, action);
        return this;
    }

    /**
     * Opens this inventory for the given player
     *
     * @param player <p>The player to open this inventory for</p>
     */
    public void openFor(Player player) {
        player.openInventory(this.inventory);
        GUIRegistry.registerOpenGUI(player, getUuid());
    }

    /**
     * Gets the action to trigger for the given inventory slot, and the given click type
     *
     * @param inventorySlot <p>The inventory slot to get the action for</p>
     * @param clickType     <p>The click type to get the action for</p>
     * @return <p>The action to run, or null if not set</p>
     */
    public GUIAction getAction(int inventorySlot, ClickType clickType) {
        Map<ClickType, GUIAction> storedActions = this.actions.get(inventorySlot);
        if (storedActions == null) {
            return null;
        } else {
            return storedActions.get(clickType);
        }
    }

}
