package net.knarcraft.knargui;

import org.bukkit.Bukkit;
import org.bukkit.entity.Player;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * A registry that keeps track of every GUI
 */
public class GUIRegistry {

    private static final Map<UUID, AbstractGUI> registeredGUIs = new HashMap<>();
    private static final Map<UUID, UUID> openGUIs = new HashMap<>();

    /**
     * Registers a GUI being opened
     *
     * @param player <p>The player the GUI is open for</p>
     * @param guiId  <p>The unique identifier for the opened GUI</p>
     */
    public static void registerOpenGUI(Player player, UUID guiId) {
        if (!registeredGUIs.containsKey(guiId)) {
            throw new IllegalArgumentException("Tried to register GUI being opened for unknown GUI");
        }
        openGUIs.put(player.getUniqueId(), guiId);
    }

    /**
     * Registers the given GUI as a tracked GUI
     *
     * @param newGUI <p>The GUI to register</p>
     */
    public static void registerGUI(AbstractGUI newGUI) {
        registeredGUIs.put(newGUI.getUuid(), newGUI);
    }

    /**
     * Gets the given player's currently open GUI
     *
     * @param player <p>The player to get the GUI for</p>
     * @return <p>The player's open GUI, or null if no GUI is open</p>
     */
    public static AbstractGUI getOpenGUI(Player player) {
        UUID guiId = openGUIs.get(player.getUniqueId());
        if (guiId == null) {
            return null;
        }

        return registeredGUIs.get(guiId);
    }

    /**
     * Registers a GUI as being closed for the given player
     *
     * @param player     <p>The player that closed a GUI</p>
     * @param autoDelete <p>Whether to immediately delete the closed GUI</p>
     */
    public static void closeGUI(Player player, boolean autoDelete) {
        //Run the close action if an anvil GUI is used
        AbstractGUI gui = getOpenGUI(player);
        if (gui instanceof AnvilGUI anvilGUI) {
            anvilGUI.runCloseAction(player);
        }
        //Un-register the player's open GUI
        openGUIs.remove(player.getUniqueId());

        if (gui != null && autoDelete) {
            deleteGUI(gui.getUuid());
        }
    }

    /**
     * Deletes the given GUI
     *
     * @param guiId <p>The id of the GUI to delete</p>
     */
    public static void deleteGUI(UUID guiId) {
        for (Player player : Bukkit.getOnlinePlayers()) {
            AbstractGUI gui = getOpenGUI(player);
            //Close the deleted GUI for every player that has it open. The listener should take care of the rest.
            if (gui != null && gui.getUuid().equals(guiId)) {
                player.closeInventory();
            }
        }
        registeredGUIs.remove(guiId);
    }

}
