package net.knarcraft.knargui.item;

import org.bukkit.enchantments.Enchantment;
import org.bukkit.inventory.ItemFlag;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.function.Consumer;

/**
 * An abstract factory for creating GUI items
 *
 * @param <B> <p>A child class implementing this class</p>
 */
@SuppressWarnings("unused")
public abstract class AbstractGUIItemFactory<B extends AbstractGUIItemFactory<B>> {

    private final ItemStack itemStack;
    private B child;

    /**
     * Instantiates a new abstract GUI item factory
     *
     * @param itemStack <p>The item stack to use for this factory</p>
     */
    protected AbstractGUIItemFactory(@NotNull ItemStack itemStack) {
        this.itemStack = itemStack;
    }

    /**
     * Sets the instance of the child class extending this class
     *
     * <p>This makes the factory return the given object. Setting this allows the factory to return a specific
     * implementation every time, without using an unsafe cast.</p>
     *
     * @param child <p>The child class extending this</p>
     */
    protected void setChild(B child) {
        this.child = child;
    }

    /**
     * Builds the item as described in previous calls
     *
     * @return <p>The output of this factory</p>
     */
    public ItemStack build() {
        return this.itemStack;
    }

    /**
     * Sets the name of this factory's item
     *
     * @param name <p>The new item name</p>
     * @return <p>The factory. Used for chaining commands</p>
     */
    public B setName(String name) {
        return changeItemMeta((meta) -> meta.setDisplayName(name));
    }

    /**
     * Sets the lore for this factory's item
     *
     * @param lore <p>The new lore</p>
     * @return <p>The factory. Used for chaining commands</p>
     */
    public B setLore(List<String> lore) {
        return changeItemMeta((meta) -> meta.setLore(lore));
    }

    /**
     * Sets whether this factory's item should look enchanted
     *
     * @param enchanted <p>True if the item should look enchanted</p>
     * @return <p>The factory. Used for chaining commands</p>
     */
    public B setEnchanted(boolean enchanted) {
        if (enchanted) {
            changeItemMeta((meta) -> meta.addEnchant(Enchantment.MENDING, 1, true));
            return changeItemMeta((meta) -> meta.addItemFlags(ItemFlag.HIDE_ENCHANTS));
        } else {
            //Remove all enchantments
            changeItemMeta((meta) -> meta.removeEnchant(Enchantment.MENDING));
            return changeItemMeta((meta) -> meta.removeItemFlags(ItemFlag.HIDE_ENCHANTS));
        }
    }

    /**
     * Changes some metadata for this factory's item
     *
     * @param action <p>The action to be run on the metadata</p>
     * @return <p>The factory. Used for chaining commands</p>
     */
    protected B changeItemMeta(Consumer<ItemMeta> action) {
        return changeItemMeta(ItemMeta.class, action);
    }

    /**
     * Changes some metadata for this factory's item
     *
     * @param metaType <p>The type of metadata to change</p>
     * @param action   <p>The action to be run on the metadata</p>
     * @param <M>      <p>The type of the metadata</p>
     * @return <p>The factory. Used for chaining commands</p>
     */
    protected <M extends ItemMeta> B changeItemMeta(Class<M> metaType, Consumer<M> action) {
        M meta = checkForNullMetadata(metaType);
        action.accept(meta);
        this.itemStack.setItemMeta(meta);
        if (child == null) {
            throw new IllegalStateException("Child class instance is not defined.");
        }
        return child;
    }

    /**
     * Throws an exception if this factory's item has null metadata
     *
     * @return <p>The metadata for this factory's item</p>
     */
    protected <M extends ItemMeta> M checkForNullMetadata(Class<M> metaType) {
        M meta = metaType.cast(this.itemStack.getItemMeta());
        if (meta == null) {
            throw new IllegalArgumentException("Cannot change missing metadata");
        } else {
            return meta;
        }
    }

}
